import java.io.*;
import java.net.*;

/**
* This class provides a means to send commands to a devserv server.  It can be run from the command line, 
* in which case it also starts a second thread running an instance of the reciever class in order to get 
* the devserv's response.
*/
public class communicator {
	private String hostname = "localhost"; // host name of computer on which devserv is running
	private int devPort = 5555; // port on which devserv server is listening
	private InetAddress myAddr = null; // InetAddress of devserv
	
	/**
	* Basic constructor that simply uses the default devserv hostname (localhost) and port (5555).
	*/
	public communicator() {
		this( "localhost", 5555 );
	}

	/**
	* Constructor that sets the devserv port and hostname to the given values
	*/
	public communicator ( String host, int port ) {
		hostname = host;
		devPort = port;
		try {
			myAddr = InetAddress.getByName(hostname);
		}
		catch(UnknownHostException e) {
			System.out.println("Unable to get IP address of host: " + hostname );
			myAddr = null;
		}
	}

	/**
	* This function sends the given message to the devserv specified.
	*
	* @return False if transmit fails
	*/
	public boolean transmit ( String msg ) {
		if( myAddr == null ) {
			System.err.println("Unable to get IP address of host: " + hostname );
			return false;
		}
		// create socket
		DatagramSocket dout = null;
		try {
			dout = new DatagramSocket();
		} catch( Exception e ) {
			System.err.println("an error occured creating a datagram socket.");
			e.printStackTrace();
			return false;
		}

		// Need to create a byte array that is packed with the string
		int len = msg.length();
		String junk = String.valueOf(len);
		int len2 = junk.length();
		int totallen = len + len2 + 1;
		byte[] msgbytes = new byte[totallen];
		String junk2 = new String(" ");

		System.arraycopy(junk.getBytes(), 0, msgbytes, 0, len2); // length of the actual message
		System.arraycopy(junk2.getBytes(), 0, msgbytes, len2, 1); // a space
		System.arraycopy(msg.getBytes(), 0, msgbytes, len2+1, len); // the actual message

		DatagramPacket packet = new DatagramPacket(msgbytes,totallen,myAddr,devPort);

		try { dout.send(packet); }
		catch(IOException e) {
			System.err.println("Error sending command!");
			e.printStackTrace();
			return false;
		}
		dout.close();
		return true;
	}

	/**
	* main() -- usage is:  communicator [devserv_hostname devserv_port] message
	* note that the message is always required (multiword messages should, of course, 
	* be enclosed in quotes) and that, the optional parameters must either both be specified
	* or both be left unspecified
	*/
	public static void main(String []args)	{
		String msg = "";
		// create the communicator
		communicator com;
		if( args.length == 1 ) {
			com = new communicator(); // use defaults!
			msg = args[0];
		} 
		else if ( args.length == 3 ) {
			String host = args[0];
			int port = Integer.parseInt(args[1]);
			com = new communicator( host, port );
			msg = args[2];
		}
		else {
			System.err.println("Usage: communicator [devserv_hostname devserv_port] message");
			return;
		}
		
      // start the listener thread
      listener myL = new listener();
      Thread myThread = new Thread( (Runnable)myL );
      myThread.start();
		
		if( com.transmit( msg ) ) { // if transmit is successful
			long start = System.currentTimeMillis();
			while( System.currentTimeMillis() - start < 10000 ) { // give the thread 10 seconds
				msg = myL.getInfo(); // get response, if its been read
				if( msg != null && msg.trim().length() > 1 ) {
					System.out.println( msg.trim() ); // output message
					break; // exit loop
				}
			}
		}
		myL.setStop(); // tell listener to stop
	}
}
