<?php
$camPower = array(1=>TRUE, TRUE); // cameras are assumed to be on

$lastDevservMsg = "";

$devserv = "localhost"; // the machine on which the devserv server is running
$devservPort = 5555; // the port on which the devserv server is listening
$errorMessage = ""; // the last error message
$response = ""; // a generic response string filled by the last command to be executed
$debug = false; // set to TRUE to have various debug statements printed

/**
* This function uses the $_POST variable passed to determine what command(s) to send.
* Pre-Condition: $_POST contains a 'request' key (and corresponding value)
* Returns: true if request was executed successfully ... false otherwise (in which case, check the error message)
* 
*/
function performRequest( $_POST ) {
   global $errorMessage, $response;
	$req = $_POST['request'];
	$errorMessage = "Unrecognized request";
	$status = false;
	if( $req == 'laptop' ) {
		$status = defaultLaptopPresentation();
	}
	else if( $req == 'desktop' ) {
		$status = defaultDesktopPresentation();
	}
	else if( $req == 'video' ) {
		$status = defaultVideoConference();
	}
	else if( $req == 'vcr' ) {
		$status = defaultVCRPlayback();
	}
	else if( $req == 'cam' ) {
		if( isset( $_POST['cmd']) && isset( $_POST['num'] ) ) {
			$cmd = $_POST['cmd'];
			$num = $_POST['num'];
			if( $cmd == "power" ) {
				$status = switchCamPower( $num );
			} else if ( $cmd == "Left" || $cmd == "Right" || $cmd == "Down" || $cmd == "Up" ) {
				$status = camMove( $cmd, $num );
			} else if ( $cmd == "In" ) {
			   $status = camZoomIn( $num );
			} else if ( $cmd == "Out" ) {
			   $status = camZoomOut( $num );
			}
			else {
			   $errorMessage = "Unrecognized camera command: $cmd";
			   return false;
			}
		}
		else {
		   $errorMessage = "Unrecognized camera command";
		   return false;
		}
	} 
	else if( $req == 'vcrPower' ) {
		$status = vcrPower();
	}
	else if( $req == 'vcrPlay' ) {
		$status = vcrPlay();
	}
	else if( $req == 'vcrRecord' ) {
		$status = vcrRecord();
	}
	else if( $req == 'vcrStop' ) {
		$status = vcrStop();
	}
	else if( $req == 'vcrEject' ) {
		$status = vcrEject();
	}
	else if( $req == 'projectorPower' ) {
		$status = projectorPower();
	}
	else if( $req == 'projectorZoomIn' ) {
		$status = projectorZoomIn();
	}
	else if( $req == 'projectorZoomOut' ) {
		$status = projectorZoomOut();
	}
	else if( $req == 'switchAudio' ) {
	   $status = switchAudio();
	}
	else if( $req == 'switchVideo' ) {
	   $status = switchVideo();
	}
	return $status;
}

/**
* This function sets the devices in the default configurations for a laptop presentation
*/
function defaultLaptopPresentation() {
   global $response, $errorMessage;
	$cmd = prependFirstLine( "vid 1 both 1 1" ); // TODO: Look up actual configuration
	if( sendToDevserv( $cmd ) ) {
		$cmd = prependFirstLine( "cam 2 pan R 10 12" ); // TODO: Figure out actual configuration for camera focused on presenter ... will need to ask about what video conference programs to see camera feed
		if( sendToDevserv( $cmd ) ) {
		    $response = "Successfully configured for laptop presenation.";
		    return true;
		}
		else {
			$errorMessage = "Error configuring camera for laptop presenation.";
			print "HI";
			return false;
		}
	}
	$errorMessage = "Error configuring videoswitch for laptop presentation.";
	return false;
}

/**
* This function sets the devices in the default configurations for a desktop presentation
*/
function defaultDesktopPresentation() {
   global $response, $errorMessage;
	$cmd = prependFirstLine( "vid 1 both 2 2" ); // TODO: Look up actual configuration
	if( sendToDevserv( $cmd ) ) {
		$cmd = prependFirstLine( "cam 2 pan R 10 12" ); // TODO: Figure out correct config ... will be same as for laptop presentation
		if( sendToDevserv( $cmd ) ) {
			$response = "Successfully configured for desktop presentation.";
			return true;
		}
		else {
			$errorMessage = "Error configuring camera for desktop presentation.";
			return false;
		}
	}
	$errorMessage = "Error configuring videoswitch for desktop presentation";
	return false;
}

/**
* This function sets the devices in the default configurations for a desktop presentation
*/
function defaultVideoConference() {
   global $response, $errorMessage;
	$cmd = prependFirstLine( "vid 1 both 3 3" ); // TODO: Look up actual configuration
	if( sendToDevserv( $cmd ) ) {
		$cmd = prependFirstLine( "cam 1 pan L 10 12" ); // TODO: Figure out correct config for camera looking at table ... is other camera needed? I assume not, but should ask
		if( sendToDevserv( $cmd ) ) {
			$response = "Successfully configured for video conference.";
			return true;
		}
		else {
		   $errorMessage = "Error configuring camera for video conference.";
		   return false;
		}
	}
	$errorMessage = "Error configuring videoswitch for video conference.";
	return false;
}

/**
* This function sets the devices in the default configurations for vcr playback ... including STARTING the tape 
* in the vcr
*/
function defaultVCRPlayback() {
   global $response, $errorMessage;
	// TODO:  Ask if this option should use the cameras
	$cmd = prependFirstLine( "vid 1 both 4 4" ); // TODO: Look up actual configuration
	if( sendToDevserv( $cmd ) ) {
		sendToSlinke( "vcr:power vcr:play" ); // turn on vcr and "press" play
		$response = "Successfully configured for vcr playback and 'pressed' play.";
		return true;
	}
	$errorMessage = "Error configuring videoswitch for vcr playback.";
	return false;
}

/**
* This function sends a power command to the vcr.  Since there seems to be no way of
* getting status from the vcr, it's impossible to determine if it was turned on or off.
*/
function vcrPower() {
   global $response, $errorMessage;
	if( sendToSlinke( "vcr:power" ) ) {
	   $response = "Sent power command to VCR.";
	   return true;
	}
	return false;
}

/**
* This function sends a play command to the vcr.
*/
function vcrPlay() {
   global $response, $errorMessage;
	if( sendToSlinke( "vcr:play" ) ) {
	   $response = "Sent play command to VCR.";
	   return true;
	}
	return false;
}

/**
* This function sends a record command to the vcr.
*/
function vcrRecord() {
   global $response, $errorMessage;
   if( sendToSlinke( "vcr:record" ) ) {
      $response = "Sent record command to VCR.";
      return true;
   }
   return false;
}

/**
* This function sends a stop command to the vcr.
*/
function vcrStop() {
   global $response, $errorMessage;
	if( sendToSlinke( "vcr:stop" ) ) {
	   $response = "Sent stop command to the VCR.";
	   return true;
	}
	return false;
}

/**
* This function sends an eject command to the vcr.
*/
function vcrEject() {
   global $response, $errorMessage;
	if( sendToSlinke( "vcr:eject" ) ) {
	   $response = "Sent eject command to the VCR.";
	   return true;
	}
	return false;
}

/**
* This function sends a power command to the projector.
*/
function projectorPower() {
   global $response, $errorMessage;
	if( sendToSlinke( "projector:power" ) ) {
	   $response = "Sent power command to the projector.";
	   return true;
	}
	return false;
}

/**
* This function sends a zoom in command to the projector.
*/
function projectorZoomIn() {
   global $response, $errorMessage;
	if( sendToSlinke( "projector:zoomin" ) ) {
	   $response = "Sent zoom in command to the projector.";
	   return true;
	}
	return false;
}

/**
* This function sends a zoom out command to the projector.
*/
function projectorZoomOut() {
   global $response, $errorMessage;
	if( sendToSlinke( "projector:zoomout" ) ) {
	   $response = "Sent zoom out command to the projector.";
	   return true;
	}
	return false;
}

/**
* This function sends the appropriate move (up, down, left, or right) to the camera (either 1 or 2).  The move direction 
* is specified in the direction paramter and should be one of the following: "Up", "Down", "Left", "Right".  
*
* To be quite honest, I'm still not entirely certain whether "panning" the camera is moving it left and right and "tilting" 
* is up and down, or if it's the opposite ... but this is what the devserv code expects :)
*/
function camMove( $direction, $cam ) {
   global $response, $errorMessage;
	if( $cam != 1 && $cam != 2 ) {
		$errorMessage = "Unrecognized camera: $camera.";
		return false;
	}
	$direction = strtolower($direction);
	$cmd = "";
	if( $direction == "up" ) {
	   $cmd = "cam $cam tilt R 1 1"; // "tilt", relative to current position, up by 1 (degree?), at a speed of 1
	} else if( $direction == "down" ) {
	   $cmd = "cam $cam tilt R -1 1"; // "tilt", relative to current position, down by 1 (degree?), at a speed of 1
	} else if( $direction == "left" ) {
	   $cmd = "cam $cam pan R -1 1"; // "pan", relative to current position, left by 1 (degree?), at a speed of 1
	} else if( $direction == "right" ) {
	   $cmd = "cam $cam pan R 1 1";  // "pan", relative to current position, left by 1 (degree?), at a speed of 1
	} else {
	   $errorMessage = "Unrecognized direction sent to camera move function.";
	   return false;
	}
	if( !sendToDevserv( prependFirstLine( $cmd ) ) ) {
	   $errorMessage = "Error attempting to move camera #$cam $direction";
	   return false;
	}
	$response = "Moved camera #$cam $direction by one degree";
	return true;
}

/**
* This function sends the appropriate command to zoom in the specified camera
*/
function camZoomIn( $cam ) {
   global $response, $errorMessage;
	if( $cam != 1 && $cam != 2 ) {
		$errorMessage = "Unrecognized camera: $cam.";
		return false;
	}
	if( !sendToDevserv( prependFirstLine( "cam $cam zoom R 1" ) ) ) {
	   $errorMessage = "Error attempting to zoom in camera #$cam";
	   return false;
	}
	$response = "Sent zoom in command to camera #$cam";
	return true;
}

/**
* This function sends the appropriate command to zoom out the specified camera
*/
function camZoomOut( $cam ) {
   global $response, $errorMessage;
	if( $cam != 1 && $cam != 2 ) {
		$errorMessage = "Unrecognized camera: $cam.";
		return false;
	}
	if( !sendToDevserv( prependFirstLine( "cam $cam zoom R -1" ) ) ) { // negative to zoom out :)
	   $errorMessage = "Error attempting to zoom out camera #$cam";
	   return false;
	}
	$response = "Sent zoom out command to camera #$cam";
	return true;
}

/**
* This function switches on (or off) the power to the specified camera.
*/
function switchCamPower( $cam ) {
   global $response, $errorMessage, $camPower;
	if( !sendToDevserv( prependFirstLine( "cam $cam power" ) ) ) { // TODO: double check whether the command needs "A on" as devserv documentation states ... I seem to remember that being wrong
	   return false;
	}
	$response = "An unexpected error occured in switchCamPower function in control_functions.php!";
	if( $camPower[$cam] ) {
		$response = "Camera " . $cam . " turned off.";
	} else {
		$response = "Camera " . $cam . " turned on.";
	}
	$camPower[$cam] = !$camPower[$cam];
	return true;
}

/**
* This function sends the appropriate commands to the devserv server to update the knox switch's audio settings
*/
function switchAudio() {
   global $response, $errorMessage;
   $response = "Connected ";
   for( $i = 1; $i < 5; $i ++ ) {
      if( $_POST['out'.$i] != $_SESSION['audioOut'.$i] )
         if( !sendToDevserv( prependFirstLine( "videoswitch 1 audio " . $_POST['out'.$i] . " " . $_SESSION['audioOut'.$i] ) ) ) {
            $errorMessage = "Error updating audio output $i to recieve from input channel " . $_POST['out'.$i];
            return false;
         } else {
            if( strlen($response) != 10 )
               $response .= ", ";
            $response .= "audio output $i with input " . $_POST['out'.$i] . " ";
         }
   }
   return true;
}

/**
* This function sends the appropriate commands to the devserv server to update the knox switch's video settings
*/
function switchVideo() {
   global $response, $errorMessage;
   $response = "Connected ";
   for( $i = 1; $i < 5; $i ++ ) {
      if( $_POST['out'.$i] != $_SESSION['videoOut'.$i] )
         if( !sendToDevserv( prependFirstLine( "videoswitch 1 video " . $_POST['out'.$i] . " " . $_SESSION['videoOut'.$i] ) ) ) {
            $errorMessage = "Error updating video output $i to recieve from input channel " . $_POST['out'.$i];
            return false;
         } else {
            if( strlen($response) != 10 )
               $response .= ", ";
            $response .= "video output $i with input " . $_POST['out'.$i];
         }
   }
   return true;
}

/**
* This function sends the given command to the slinke device by executing the provided command line 
* interface tool (slcmd)
* Returns true if command executed successfully.
*/
function sendToSlinke( $cmd ) {
   global $response, $errorMessage;
	//system( "slcmd send \"$cmd\"" ); // make sure to enclose cmd in quotes in case it has spaces
	$errorMessage = "slinke commands disabled since the software isn't installed -- and wouldn't work anyway since I don't have the slinke";
	return false;
}

/**
* This function sends the devserv a command asking it to reply with the current status information.
*/
function getStatus() {
   global $response, $errorMessage;
   if( sendToDevserv( prependFirstLine( "devserv description" ) ) ) {
      $response = "Got current status of devices.";
      return true;
   }
   $errorMessage = "Error getting status of devices from server!";
   return false;
}

/*
* This function takes a message, sends it, and gets the response by executing a simple java application ... not 
* all that elegant a solution ... at some point the devserv code should be rewritten
* Also note that this function assumes that the message is complete (ie: it doesn't try to prepend the first
* line to it).
* The devserv's response is stored in the global $lastDevservMsg.
* Returns true if everything worked, or false is something didn't (ie: a null or empty string from devserv)
*/
function sendToDevserv( $msg ) {
	global $devserv, $devservPort, $errorMessage, $lastDevservMsg;
	$lastDevservMsg = "";
	
	// use java code to transmit message
	$cmd = "java -cp \"C:\Program Files\Apache Group\Apache2\htdocs\" communicator $devserv $devservPort \"$msg\"";
	//print "executing: <br> " . $cmd . "<br>\n";
	$response = shell_exec( $cmd );
	
	$lastDevservMsg = $response; // store the message
	if( $response != null && strlen($response) > 5 )
	   return true;
	return false;
}
/*
* This function takes the message to be sent to the devserv server and prepends the
* appropriate first "line" to the message.  Note that this should be done last since
* the first line contains states the size of the message.
*/
function prependFirstLine ( $line ) {
	global $devserv;
	$line = date( "U" ) . " $devserv#$line";
	$line = strlen( $line ) . " $line";
	return $line;
}

?>