<?php
session_start();
require_once "HTML/Table.php";
require_once "HTML/Form.php";

print createStart();

if( !isset( $_SESSION['login'] ) ) {
   header("Location: main.php"); /* Redirect browser */
   exit;
}


$status = 0; // if status is true, then the _SESSION array contains the relevant status information
if( isset( $_SESSION['cam1']) ) { // assume that if this one is defined all the others are as well
	$status = 1;
} else { // if no current state is known, set defaults to invalid values
	$_SESSION['cam1'] = 0; // not really invalid, just "off"
	$_SESSION['cam2'] = 0; // not really invalid, just "off"
	for( $i = 1; $i < 5; $i ++ ) {
		$_SESSION[ "audio" . $i ] = 5; // invalid!
		$_SESSION[ "video" . $i ] = 5; // invalid!
	}
}

// create form variable ... just the basics that all of the forms will use
$form = new HTML_Form("status.php","POST"); // NOTE: the target for this form is the message frame (set in header info)

// create table containing all the basic control options
$table = new HTML_Table();
$table -> setRowCount( 1 );
$table -> setColCount( 7 );
// put in basic controls
$table -> setCellContents( 0, 0, "<h3>Typical<br>Configurations:</h3>" );
$table -> setCellContents( 0, 3, generateFormCode( "laptop", "Laptop Presentation" ) );
$table -> setCellContents( 0, 4, generateFormCode( "desktop", "Desktop Presentation" ) );
$table -> setCellContents( 0, 5, generateFormCode( "video", "Video Conference" ) );
$table -> setCellContents( 0, 6, generateFormCode( "vcr", "VCR Playback" ) );

// output table
echo $table -> toHTML();

// create table containing all the manual control options
$table = new HTML_Table();
$table -> setRowCount(11);
$table -> setColCount(7);
// add manual controls text
$table -> setCellContents( 0, 0, "<h3>Manual Control Options (Experts Only)</h3>" );
$table -> setCellAttributes( 0, 0, "colspan=10" );
// add manual Projector controls
$table -> setCellContents( 1, 0, "<h4>Projector:</h4>" );
$table -> setCellContents( 2, 1, generateFormCode( "projectorPower", "Power" ) );
$table -> setCellContents( 3, 1, generateFormCode( "projectorZoomIn", "Zoom In" ) );
$table -> setCellContents( 3, 2, generateFormCode( "projectorZoomOut", "Zoom Out" ) );
// add manual VCR controls
$table -> setCellContents( 1, 6, "<h4>VCR:</h4>" );
$table -> setCellContents( 2, 7, generateFormCode( "vcrPower", "Power" ) );
$table -> setCellContents( 2, 8, generateFormCode( "vcrPlay", "Play" ) );
$table -> setCellContents( 3, 7, generateFormCode( "vcrRecord", "Record" ) );
$table -> setCellContents( 3, 8, generateFormCode( "vcrStop", "Stop" ) );
$table -> setCellContents( 4, 8, generateFormCode( "vcrEject", "Eject" ) );
// add manual camera controls
$table -> setCellContents( 6, 0, "<h4>Camera:</h4>" );
$table -> setCellContents( 7, 1, generateCameraControlCode() );
$table -> setCellAttributes( 7, 1, "colspan=5 rowspan=5" );
// add manual Video Switch Controls
$table -> setCellContents( 6, 6, "<h4>Video Switch:</h4>" );
$table -> setCellContents( 7, 7, generateVideoSwitchChannelCode( "Audio" ) );
$table -> setCellAttributes( 7, 7, "rowspan=5" );
$table -> setCellContents( 7, 9, generateVideoSwitchChannelCode( "Video" ) );
$table -> setCellAttributes( 7, 9, "rowspan=5" );

echo $table -> toHTML();

print createEnd();

/**********************************************************
End of output section, beginning of function definitions
**********************************************************/


/**
* This function returns a string containing the HTML code for a form containing a hidden field (with name request and 
* $hiddenValue as value), a submit button (with value $submitValue), and, optionally, a second hidden field with the 
* specified name and value.
* NOTE:  This function uses the global variable $form (an instance of HTML_Form) to generate the code and assumes that 
* the $form variable was initiallized with the appropriate attributes for the form tag.
*/
function generateFormCode( $hiddenValue, $submitValue, $hiddenName2="", $hiddenValue2="" ) {
	global $form;
	$code = $form -> returnStart(); // the form tag itself
	$code .= $form -> returnHidden( "request", $hiddenValue ); // the hidden form saying what change was requested
	if( $hiddenName2 != "" ) { // if a second hidden form was requested
		$code .= $form -> returnHidden( $hiddenName2, $hiddenValue2 );
	}
	$code .= $form -> returnSubmit( $submitValue ); // the submit tag
	$code .= "</form>"; // the end of the form tag
	return $code;
}

/**
* This function creates the table (adn the form in which the table resides) containing the various controls for the cameras.
* The form will return the value "cam" for the "request", and then the appropriate command for the field "cmd" (ex: "power", 
* "zoomIn").  The camera selected to be controlled is specified in "num" and will be either 1 or 2.
*
* Note: This function uses the global variable $form
*/
function generateCameraControlCode( ) {
	global $form;
	$code = $form -> returnStart(); // form tag itself
	$code .= $form -> returnHidden( "request", "cam" );
	// make table
	$table = new HTML_Table();
	$table -> setRowCount( 7 );
	$table -> setColCount( 6 );
	// add camera select
	$table -> setCellContents( 0, 0, "<b>Select Camera:</b> " );
	$table -> setCellAttributes( 0, 0, "colspan=3" );
	$table -> setCellContents( 0, 3, "<select name=num><option>1<option>2</select>" );
	// add power command
	$table -> setCellContents( 2, 1, $form -> returnSubmit( "power", "cmd" ) );
	$table -> setCellAttributes( 2, 1, "colspan=2" );
	// add zoom commands
	$table -> setCellContents( 6, 0, "<b>Zoom:</b>" );
	$table -> setCellContents( 7, 1, $form -> returnSubmit( "In", "cmd" ) );
	$table -> setCellContents( 7, 2, $form -> returnSubmit( "Out", "cmd" ) );
	// add move commands -- the "pan" (left and right?) and "tilt" (up and down?) ... or so I'm guessing
	$table -> setCellContents( 2, 5, $form -> returnSubmit( "Up", "cmd" ) );
	$table -> setCellAttributes( 2, 5, "align=center" );
	$table -> setCellContents( 3, 4, $form -> returnSubmit( "Left", "cmd" ) );
	$table -> setCellAttributes( 3, 4, "align=right" );
	$table -> setCellContents( 3, 5, "<b>Move</b>" );
	$table -> setCellContents( 3, 6, $form -> returnSubmit( "Right", "cmd" ) );
	$table -> setCellContents( 4, 5, $form -> returnSubmit( "Down", "cmd" ) );
	$table -> setCellAttributes( 4, 5, "align=center" );
	
	$code .= $table -> toHTML();
	$code .= "</form>";
	return $code;
}

/**
*  This function creates the table (and the form in which the table resides) containing the radio buttons for setting 
* the input and output channels of either the audio or video channel of the videoswitch.  The table includes the 
* appropriate labels for each axis.  Note that, in generating the form code, the global variable $form is used.
*
*  The "type" parameter passed should be a string (either "audio" or "video") specifing the type of channel.  If the 
* parameter has an unexpected value, an error string is returned.
*/
function generateVideoSwitchChannelCode( $type ) {
	if( $type != "Audio" && $type != "Video" ) {
		return "Error, unknown type, $type, sent to generateVideoSwitchChannelCode function.";
	}
	global $form;
	// generate start of form code
	$code = $form -> returnStart(); // form tag itself
	$code .= $form -> returnHidden( "request", "switch".$type ); // hidden form saying whether change(s) are for video or audio channel
	
	$attr = array ( "align" => "center" );
	$table = new HTML_Table($attr);
	$table -> setRowCount( 7 );
	$table -> setColCount( 6 );
	// put in "Input" label
	$table -> setCellContents( 0, 2, "<b>$type Input</b>" );
	$table -> setCellAttributes( 0, 2, "colspan=4" );
	// put in Input channel numbers
	$table -> setCellContents( 1, 2, "<b>1</b>" );
	$table -> setCellContents( 1, 3, "<b>2</b>" );
	$table -> setCellContents( 1, 4, "<b>3</b>" );
	$table -> setCellContents( 1, 5, "<b>4</b>" );
	// put in "Output" label
	$table -> setCellContents( 2, 0, "<b>O&nbsp<br>u<br>t<br>p<br>u<br>t</b>" );
	$table -> setCellAttributes( 2, 0, "rowspan=4" );
	// put in Output channel numbers
	$table -> setCellContents( 2, 1, "<b>1</b>" );
	$table -> setCellContents( 3, 1, "<b>2</b>" );
	$table -> setCellContents( 4, 1, "<b>3</b>" );
	$table -> setCellContents( 5, 1, "<b>4</b>" );
	// put in radio buttons ... the HTML_Form class doesn't have a returnRadio function, so do this manually, sigh :)
	$checked = "";
	$type = strtolower( $type ); // for use with $_SESSION array
	$table -> setCellContents( 2, 2, generateRadioInputCode( "out1", "1", ($_SESSION[ $type . 'Out1' ] == 1)) );
	$table -> setCellContents( 2, 3, generateRadioInputCode( "out1", "2", ($_SESSION[ $type . 'Out1' ] == 2)) );
	$table -> setCellContents( 2, 4, generateRadioInputCode( "out1", "3", ($_SESSION[ $type . 'Out1' ] == 3)) );
	$table -> setCellContents( 2, 5, generateRadioInputCode( "out1", "4", ($_SESSION[ $type . 'Out1' ] == 4)) );
	$table -> setCellContents( 3, 2, generateRadioInputCode( "out2", "1", ($_SESSION[ $type . 'Out2' ] == 1)) );
	$table -> setCellContents( 3, 3, generateRadioInputCode( "out2", "2", ($_SESSION[ $type . 'Out2' ] == 2)) );
	$table -> setCellContents( 3, 4, generateRadioInputCode( "out2", "3", ($_SESSION[ $type . 'Out2' ] == 3)) );
	$table -> setCellContents( 3, 5, generateRadioInputCode( "out2", "4", ($_SESSION[ $type . 'Out2' ] == 4)) );
	$table -> setCellContents( 4, 2, generateRadioInputCode( "out3", "1", ($_SESSION[ $type . 'Out3' ] == 1)) );
	$table -> setCellContents( 4, 3, generateRadioInputCode( "out3", "2", ($_SESSION[ $type . 'Out3' ] == 2)) );
	$table -> setCellContents( 4, 4, generateRadioInputCode( "out3", "3", ($_SESSION[ $type . 'Out3' ] == 3)) );
	$table -> setCellContents( 4, 5, generateRadioInputCode( "out3", "4", ($_SESSION[ $type . 'Out3' ] == 4)) );
	$table -> setCellContents( 5, 2, generateRadioInputCode( "out4", "1", ($_SESSION[ $type . 'Out4' ] == 1)) );
	$table -> setCellContents( 5, 3, generateRadioInputCode( "out4", "2", ($_SESSION[ $type . 'Out4' ] == 2)) );
	$table -> setCellContents( 5, 4, generateRadioInputCode( "out4", "3", ($_SESSION[ $type . 'Out4' ] == 3)) );
	$table -> setCellContents( 5, 5, generateRadioInputCode( "out4", "4", ($_SESSION[ $type . 'Out4' ] == 4)) );
	// put in submit button so as to make it centered
	$table -> setCellContents( 6, 2, $form -> returnSubmit( "Apply" ) );
	$table -> setCellAttributes( 6, 2, "colspan=4" );
	
	// add table code and form end tag
	$code .= $table -> toHTML();
	$code .= "</form>";
	
	return $code;
}

/**
* This function returns the HTML code for a radio input with the given name and value.  If the 
* $checked variable is true then the radio input is set to be checked.
*/
function generateRadioInputCode( $name, $value, $checked ) {
	$code = "<input type=radio";
	if( $checked )
		$code .= " checked";
	$code .= " name=$name value=$value>";
	return $code;
}

/**
* Simple function that returns the string representing the start of the actual HTML code
*/
function createStart() {
   return "<html>\n<head>\n<title>Remote Setup</title>\n<base target=\"message\">\n</head>\n<body>\n";
}

/**
* Simple function that returns the string representing the end of the actual HTML code
*/
function createEnd() {
   return "</body>\n</html>\n";
}

?>