import java.io.*;
import java.net.*;

/**
* This class is meant to be run by a seperate thread in order to continuously listen for 
* messages from the devserv server.
*/
public class listener implements Runnable{
	private String multicastIP = "224.0.1.1"; // the multicast IP on which to listen
	private int multiPort = 5555; // the multicast port on which to listen
	private MulticastSocket mySocket = null; // the connection to the multicast ip:port
	
	private String myMessage = ""; // the last actual message read
	
	private boolean connected = false; // boolean stating whether multicastsocket is connected
	private boolean stop = false; // boolean stating whether to stop thread
	

	/**
	* Basic constructor that uses the default multicast ip (224.0.1.1) and port (5556)
	*/
	public listener() {
	}
	
	/**
	* Constructor that sets the multicast ip and port to the passed values.
	*/
	public listener( String multiIP, int port ) {
		multicastIP = multiIP;
		multiPort = port;
	}

	public void run() {
		// make the actual connection
		makeConnection();
		
		// perform loop to listen for messages
		while (connected && !stop) {
			readDevString();
		}
		if ( connected ) 
			mySocket.close();
	}
	
	public void setStop() {
		stop = true;
	}
	
	private void makeConnection() {
		// connect to multicast socket
		InetAddress addr;
		try {
			addr = InetAddress.getByName( multicastIP );
		} catch( Exception e ) {
			System.out.println("error getting hostname");
			e.printStackTrace();
			return;
		}
		try {
			mySocket = new MulticastSocket(multiPort+1);
			mySocket.joinGroup(addr);
			mySocket.setSoTimeout(100);
		} catch( Exception e ) {
			System.err.println("Error receiving message");
			e.printStackTrace();
			return;
		}
		connected = true;
	}
	
	/** Read the status or description messages sent by the devserv server
	*/
	private int readDevString() {
		DatagramPacket msg;
		try {
			msg = new DatagramPacket(new byte[1024], 1024);
		} catch( Exception e ) {
			System.err.println("error trying to create datagram packet:");
			e.printStackTrace();
			return -1;
		}
		try {
			mySocket.receive(msg);
		} catch( SocketTimeoutException t ) {
			// don't do anything, timeouts are okay
		} catch( Exception e ) {
			System.err.println("Error receiving message");
			e.printStackTrace();
			return -1;
		}
		String message = new String( msg.getData() );
		if( message != null && message.length() != 0 ) {
			myMessage = message;
		}
		return 0;
	}
	
	/**
	*  Returns the last unseen message ... returns an empty string if no unread messages exist
	*/
	public String getInfo() {
		String msg = myMessage;
		myMessage = "";
		return msg;
	}
	
}