<?php
include_once "control_functions.php";
session_start(); 

if( !isset( $_SESSION['login'] ) ) {
   header("Location: main.php"); /* Redirect browser */
   exit;
}

print writeStart();
$success = false;
$forceReload = false; // if true, then we want to force reloading of the controls frame regardless 
                     // of the success/failure of communications
// get request, if one was issued
if( isset( $_POST['request']) ) {
   $success = performRequest( $_POST );
} else { // page loaded without a request so get description from devserv
   $success = getStatus();
   if( $success ) {
      $forceReload = true;  // since there was no request we assume that this a new load and so want to redisplay the controls
   }
}

if( $forceReload || ($success && parseDevservResponse( $lastDevservMsg )) ) { // check if gui.php file needs to be reloaded
   print "<script>window.parent.controls.location = \"gui.php\";</script>";
}

print "<b>Response: </b> ";
if( $success ) {
   print $response;
}
else {
   print $errorMessage;
}
print "<br>\n";
print "<b>Status:</b> Ready<br>\n";

print "\n</body>\n</html>\n";

/**
* This function parses the given devserv message, taking the relevant device information from it and storing it in the _SESSION array.
* If the value stored is different from the value there previously (or if there was no value previously), then the function returns true.
*/
function parseDevservResponse( $msg ) {
   $line = strtok( $msg, "#" ); // first line is the message time stamp, etc info ... ignore
   $line = strtok( "#" ); // second line is the message type ... ignore
   $line = strtok( "#" ); // third line is the devserv info ... ignore
   $line = strtok( "#" ); // ignore fourth line too
   $line = strtok( "#" ); // ...
   $changed = false;
   for( $line = strtok( "#" ); $line !== FALSE; $line = strtok( "#" ) ) { // all subsequent lines are descriptions of the devices
      //print "reading line: $line<br>";
      if( strncmp( $line, "camera", 6 ) == 0 ) { // a camera device
         if( parseCameraStatus( $line ) )
            $changed = true;
      } else if( strncmp( $line, "videoswitch", 11 ) == 0 ) { // a videoswitch device
         if( parseVideoSwitchStatus( $line ) )
            $changed = true;
      }
   }
   return $changed;
}

/**
* This function parses the given line which is assumed to be a line from a devserv server message containing the status info of a camera.  
* Currently, the function only works for Sony EVI-D30 cameras
* Returns: true camera status has changed, false if it hasn't
*/
function parseCameraStatus( $line ) {
   $pos = strpos( $line, "\"" );
   if( $pos != true )
      return false; // unexpected format
   $pos ++;
   $camera = substr( $line, $pos, strpos( $line, "\"", $pos ) - $pos );
   $num = substr( $line, 7, 1 );
   if( strcmp( $camera, "Sony EVI-D30" ) != 0 )
      return false; // not the right camera type, so just return false
   // there is only one thing we actually care about for cameras ... whether they're on or off
   $line = substr( $line, strrpos( $line, "power " )+1 ); // drop everything before "power "
   if( strncmp( $line, "on", 2) == 0 )
      $changed = addToArray( "cam" . $num, 0, $_SESSION );  // $array[1] is camera number ... 0 is "off"
   else
      $changed = addToArray( "cam" . $num, 1, $_SESSION );  // $array[1] is camera number ... 1 is "on"
   
   return $changed;
}

/**
* This function parses the given line (assumed to be a line from a devserv server message, containing the status info of a video switch).  
* Currently, the function only works for Knox Video RS4X4HB video switches
* Returns true if video switch status has changed
*/
function parseVideoSwitchStatus( $line ) {
   $pos = strpos( $line, "\"" );
   if( $pos != true )
      return false; // unexpected format
   $pos ++;
   $switch = substr( $line, $pos, strpos( $line, "\"", $pos ) - $pos );
   if( strcmp( $switch, "KNOX VIDEO RS4X4HB" ) != 0 )
      return false; // wrong video switch type
   $changed = false;
   $pos = strpos( $line, "output" );
   while( $pos !== FALSE ) {
      $line = substr( $line, $pos );
//      print "line is: $line<br>";
      if( !addToArray( "videoOut" . substr( $line, 7, 1 ), substr( $line, strpos($line, "Video") + 6, 1 ), $_SESSION ) )
         $changed = true;
      if( !addToArray( "audioOut" . substr( $line, 7, 1 ), substr( $line, strpos($line, "Audio") + 6, 1 ), $_SESSION ) )
         $changed = true;
      $pos = strpos( $line, "output", 7 );
   }
}

/**
* This helper functions places the given key & value pair into the specified array.  It returns true if this action changed the 
* array (ie: the key hadn't existed before, or had had a different value).  The function returns false if the action would not have 
* effected the array (and, as a side note, doesn't bother to do the actual insert because of this).
*/
function addToArray( $key, $value, &$array ) {
   $changed = false;
   if( !isset( $array[ $key ] ) )
      $changed = true;
   else {
      if( $array[ $key ] != $value )
         $changed = true;
   }
   //print "setting $key to $value<br>";
   $array[ $key ] = $value;
   return $changed;
}

/**
*  This function returns a string containing the html code for the start of the html page (up to, and including, the <body> tag).
*/
function writeStart() {
   $html = "<html>\n";
   $html .= "<head>\n";
   $html .= "<title>Status</title>\n";
   $html .= "</head>\n";
   $html .= "<body>\n";
   return $html;
}

?>